@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2014  Winfried Bruns, Bogdan Ichim, Christof Soeger
+ * Copyright (C) 2007-2019  Winfried Bruns, Bogdan Ichim, Christof Soeger
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -21,8 +21,8 @@
  * terms of service.
  */
 
-#ifndef CONE_PROPERTY_H_
-#define CONE_PROPERTY_H_
+#ifndef LIBNORMALIZ_CONE_PROPERTY_H_
+#define LIBNORMALIZ_CONE_PROPERTY_H_
 
 #include <bitset>
 #include <ostream>
@@ -35,77 +35,111 @@ namespace libnormaliz {
  */
 namespace ConeProperty {
     enum Enum {
-        //
-        // goals that can be computed (or are defined by input data)
-        //
-        // matrix valued
-        Generators,
+        FIRST_MATRIX,
+        Generators = ConeProperty::FIRST_MATRIX,
         ExtremeRays,
-        VerticesFloat,
         VerticesOfPolyhedron,
         SupportHyperplanes,
-        SuppHypsFloat,
         HilbertBasis,
         ModuleGenerators,
         Deg1Elements,
+        LatticePoints,
         ModuleGeneratorsOverOriginalMonoid,
-        Sublattice, 
         ExcludedFaces,
         OriginalMonoidGenerators,
         MaximalSubspace,
         Equations,
         Congruences,
-        //vector valued
-        Grading,
+        LAST_MATRIX = ConeProperty::Congruences,
+        FIRST_MATRIX_FLOAT,
+        SuppHypsFloat = ConeProperty::FIRST_MATRIX_FLOAT,
+        VerticesFloat,
+        LAST_MATRIX_FLOAT = ConeProperty::VerticesFloat,
+        // Vector values
+        FIRST_VECTOR,
+        Grading = ConeProperty::FIRST_VECTOR,
         Dehomogenization,
         WitnessNotIntegrallyClosed,
         GeneratorOfInterior,
-        // Cardinalities
-        TriangulationSize,
-        // Integer valued,        
-        TriangulationDetSum,
+        ClassGroup,
+        LAST_VECTOR = ConeProperty::ClassGroup,
+        // Integer valued,
+        FIRST_INTEGER,
+        TriangulationDetSum = ConeProperty::FIRST_INTEGER,
         ReesPrimaryMultiplicity,
         GradingDenom,
         UnitGroupIndex,
         InternalIndex,
-        ExternalIndex,
+        LAST_INTEGER = ConeProperty::InternalIndex,
+        FIRST_GMP_INTEGER,
+        ExternalIndex = FIRST_GMP_INTEGER,
+        LAST_GMP_INTEGER = ConeProperty::ExternalIndex,
         // rational valued
-        Multiplicity,
+        FIRST_RATIONAL,
+        Multiplicity = ConeProperty::FIRST_RATIONAL,
         Volume,
         Integral,
         VirtualMultiplicity,
+        LAST_RATIONAL = ConeProperty::VirtualMultiplicity,
+        // field valued
+        FIRST_FIELD_ELEM,
+        RenfVolume=FIRST_FIELD_ELEM,
+        LAST_FIELD_ELEM=ConeProperty::RenfVolume,
         // floating point valued
-        EuclideanVolume,
+        FIRST_FLOAT,
+        EuclideanVolume = ConeProperty::FIRST_FLOAT,
+        EuclideanIntegral,
+        LAST_FLOAT = ConeProperty::EuclideanIntegral,
         // dimensions
+        FIRST_MACHINE_INTEGER,
+        TriangulationSize = ConeProperty::FIRST_MACHINE_INTEGER,
+        NumberLatticePoints,
         RecessionRank,
         AffineDim,
         ModuleRank,
         Rank,
-        EmbeddingDim,      
+        EmbeddingDim,
+        LAST_MACHINE_INTEGER = ConeProperty::EmbeddingDim,
         // boolean valued 
-        IsPointed,
+        FIRST_BOOLEAN,
+        IsPointed = ConeProperty::FIRST_BOOLEAN,
         IsDeg1ExtremeRays,
         IsDeg1HilbertBasis,
         IsIntegrallyClosed,
         IsReesPrimary,
         IsInhomogeneous,
-        IsGorenstein,        
+        IsGorenstein,
+        LAST_BOOLEAN = ConeProperty::IsGorenstein,
         // complex structures
-        Triangulation,
+        FIRST_COMPLEX_STRUCTURE,
+        Triangulation = ConeProperty::FIRST_COMPLEX_STRUCTURE,
         StanleyDec,
-        InclusionExclusionData,        
-        ClassGroup,        
+        InclusionExclusionData,
         IntegerHull,
         ProjectCone,
         ConeDecomposition,
+        //
+        Automorphisms,
+        AmbientAutomorphisms,
+        CombinatorialAutomorphisms,
+        RationalAutomorphisms,
+        EuclideanAutomorphisms,
+        //
         HilbertSeries,
         HilbertQuasiPolynomial,
+        EhrhartSeries,
+        EhrhartQuasiPolynomial,
         WeightedEhrhartSeries,
-        WeightedEhrhartQuasiPolynomial,        
+        WeightedEhrhartQuasiPolynomial,
+        FaceLattice,
+        FVector,
+        Sublattice,
+        LAST_COMPLEX_STRUCTURE = ConeProperty::Sublattice,
         //
         // integer type for computations
         //
-        BigInt,
+        FIRST_PROPERTY,
+        BigInt = ConeProperty::FIRST_PROPERTY,
         //
         // algorithmic variants
         //
@@ -130,6 +164,13 @@ namespace ConeProperty {
         NoRelax,
         Descent,
         NoDescent,
+        NoGradingDenom,
+        GradingIsPositive,
+        ExploitAutomsVectors,
+        ExploitAutomsMult,
+        //
+        Dynamic,
+        Static,
         //
         // checking properties of already computed data
         // (cannot be used as a computation goal)
@@ -141,11 +182,28 @@ namespace ConeProperty {
         //
         ExplicitHilbertSeries,
         NakedDual,
-        
-        EnumSize // this has to be the last entry, to get the number of entries in the enum
+        EnumSize,
+        LAST_PROPERTY = ConeProperty::EnumSize // this has to be the last entry, to get the number of entries in the enum
     }; // remember to change also the string conversion function if you change this enum
 }
 
+namespace OutputType{
+    enum Enum {
+        Matrix,
+        MatrixFloat,
+        Vector,
+        Integer,
+        GMPInteger,
+        Rational,
+        FieldElem,
+        Float,
+        MachineInteger,
+        Bool,
+        Complex,
+        Void
+    };
+}
+
 class ConeProperties {
 public:
     /* Constructors */
@@ -178,10 +236,13 @@ public:
     ConeProperties options();
 
     /* the following methods are used internally */
-    void set_preconditions(bool inhomogeneous);    // activate properties which are needed implicitly
-    void prepare_compute_options(bool inhomogeneous);
+    void set_preconditions(bool inhomogeneous, bool numberfield);    // activate properties which are needed implicitly
+    // void prepare_compute_options(bool inhomogeneous, bool numberfield);
     void check_sanity(bool inhomogeneous);
+
     void check_conflicting_variants();
+    void check_Q_permissible(bool after_implications);
+    // void set_default_goals(bool inhomogeneous, bool numberfield);
 
     /* print it in a nice way */
     friend std::ostream& operator<<(std::ostream&, const ConeProperties&);
@@ -197,6 +258,7 @@ bool isConeProperty(ConeProperty::Enum& cp, const std::string& s);
 ConeProperty::Enum toConeProperty(const std::string&);
 const std::string& toString(ConeProperty::Enum);
 std::ostream& operator<<(std::ostream&, const ConeProperties&);
+OutputType::Enum output_type(ConeProperty::Enum);
 
 }
 
