@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2014  Winfried Bruns, Bogdan Ichim, Christof Soeger
+ * Copyright (C) 2007-2019  Winfried Bruns, Bogdan Ichim, Christof Soeger
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -22,8 +22,8 @@
  */
 
 //---------------------------------------------------------------------------
-#ifndef MATRIX_HPP
-#define MATRIX_HPP
+#ifndef LIBNORMALIZ_MATRIX_HPP
+#define LIBNORMALIZ_MATRIX_HPP
 //---------------------------------------------------------------------------
 
 
@@ -32,14 +32,14 @@
 #include <iostream>
 #include <string>
 
-#include <libnormaliz/libnormaliz.h>
+#include <boost/dynamic_bitset.hpp>
+
+#include <libnormaliz/general.h>
 #include <libnormaliz/integer.h>
 #include <libnormaliz/convert.h>
 #include <libnormaliz/vector_operations.h>
 // #include <libnormaliz/sublattice_representation.h>
 
-#include <boost/dynamic_bitset.hpp>
-
 //---------------------------------------------------------------------------
 
 namespace libnormaliz {
@@ -100,7 +100,7 @@ template<typename Integer> class Matrix {
          Integer& denom, bool ZZ_invertible, bool transpose, size_t red_col, size_t sign_col,
          bool compute_denom=true, bool make_sol_prime=false);
                     
-    size_t row_echelon_inner_elem(bool& success); // does the work and checks for overflows
+    // size_t row_echelon_inner_elem(bool& success); // does the work and checks for overflows
     // size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     // NOTE: Bareiss cannot be used if z-invertible transformations are needed
     
@@ -115,7 +115,7 @@ template<typename Integer> class Matrix {
     // computes rank and index simultaneously, returns rank
     Integer full_rank_index(bool& success);
     
-    vector<key_t> max_rank_submatrix_lex_inner(bool& success) const;
+    vector<key_t> max_rank_submatrix_lex_inner(bool& success, vector<key_t> perm=vector<key_t>(0)) const;
     
     // A version of invert that circumvents protection and leaves it to the calling routine
     Matrix invert_unprotected(Integer& denom, bool& sucess) const;
@@ -150,7 +150,9 @@ template<typename Integer> class Matrix {
                     
 public:
 
-size_t row_echelon_inner_bareiss(bool& success, Integer& det);
+    typedef Integer elem_type;
+
+    size_t row_echelon_inner_bareiss(bool& success, Integer& det);
 
     vector<vector<Integer>* > submatrix_pointers(const vector<key_t>& key);
     
@@ -204,10 +206,16 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
 
     void swap (Matrix<Integer>& x);
 
-	// returns the permutation created by sorting the rows with a grading function
+    // returns the permutation created by sorting the rows with a grading function
     // or by 1-norm if computed is false
     vector<key_t> perm_sort_by_degree(const vector<key_t>& key, const vector<Integer>& grading, bool computed) const;
+    // according to the matrix of weights (taking absolute values first)
     vector<key_t> perm_by_weights(const Matrix<Integer>& Weights, vector<bool> absolute);
+    // according to the number of zeoes -- descending -- 
+    vector<key_t> perm_by_nr_zeroes();
+    // according to lex order
+    vector<key_t> perm_by_lex();
+    
     
     void select_submatrix(const Matrix<Integer>& mother, const vector<key_t>& rows);
     void select_submatrix_trans(const Matrix<Integer>& mother, const vector<key_t>& rows);
@@ -215,10 +223,7 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     Matrix& remove_zero_rows(); // remove zero rows, modifies this
 
     // resizes the matrix to the given number of rows/columns
-    // if only the number of rows shrinks it will keep all its allocated memory
-    // but with delete_rows=true they will really be deleted 
-    // useful when the size varies
-    void resize(size_t nr_rows, bool delete_rows=false);
+    void resize(size_t nr_rows);
     void resize(size_t nr_rows, size_t nr_cols);
     void resize_columns(size_t nr_cols);
     void Shrink_nr_rows(size_t new_nr_rows);
@@ -234,6 +239,8 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     void append(const vector<vector<Integer> >& M); // the same, but for another type of matrix
     void append(const vector<Integer>& v); // append the row v to this
     void append_column(const vector<Integer>& v); // append the column v to this
+    void insert_column(const size_t pos,const vector<Integer>& v);
+    void insert_column(const size_t pos,const Integer& val);
     void remove_row(const vector<Integer>& row); // removes all appearances of this row, not very efficient!
     void remove_row(const size_t index);
     vector<size_t> remove_duplicate_and_zero_rows();
@@ -243,7 +250,7 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     inline const Integer& get_elem(size_t row, size_t col) const {
         return elem[row][col];
     }
-    inline const vector< vector<Integer> >& get_elements() const {
+    inline const vector< vector<Integer> >& get_elements() const{
         assert(nr==elem.size());
         return elem;
     }
@@ -263,6 +270,10 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     //  convert the remaining matrix to nmz_float
     Matrix<nmz_float> nmz_float_without_first_column() const;
     
+    void make_first_element_1_in_rows();
+    void standardize_basis();
+    void standardize_rows(const vector<Integer>& Norm);
+    void standardize_rows();
     
 
 //---------------------------------------------------------------------------
@@ -271,10 +282,14 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
 
     Matrix add(const Matrix& A) const;                       // returns this+A
     Matrix multiplication(const Matrix& A) const;          // returns this*A
+    void multiplication(Matrix& B, const Matrix& A) const; // the same, but result in B
+    Matrix multiplication_trans(const Matrix& A) const;          // returns this*A.transpose()
+    void multiplication_trans(Matrix& B, const Matrix& A) const; // the same, but result in B
     Matrix multiplication(const Matrix& A, long m) const;// returns this*A (mod m)
     bool equal(const Matrix& A) const;             // returns this==A
     // bool equal(const Matrix& A, long m) const;     // returns this==A (mod m)
     Matrix transpose() const;                     // returns the transpose of this
+    void transpose_in_place();
     
     bool is_diagonal() const;
 
@@ -291,6 +306,7 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
                                           // vector of gcds returned
     void make_cols_prime(size_t from_col, size_t to_col);   
              // the columns of this in the specified range are reduced by their gcd
+    void simplify_rows(const vector<Integer>& Norm); // applies v_standardize to the rows
 
     Matrix multiply_rows(const vector<Integer>& m) const;  //returns matrix were row i is multiplied by m[i]
 
@@ -302,6 +318,8 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
    vector<Integer> MxV(const vector<Integer>& v) const;//returns this*V
    vector<Integer> VxM(const vector<Integer>& v) const;//returns V*this
    vector<Integer> VxM_div(const vector<Integer>& v, const Integer& divisor,bool& success) const; // additionally divides by divisor
+   
+   bool check_congruences(const vector<Integer>& v) const; // *this represents congruences
 
 //---------------------------------------------------------------------------
 //                          Matrix operations
@@ -310,6 +328,8 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
 
 // Normal forms
 
+size_t row_echelon_inner_elem(bool& success); // does the work and checks for overflows
+
 // converts this to row echelon form over ZZ and returns rank, GMP protected, uses only elementary transformations over ZZ
     size_t row_echelon();
 
@@ -341,7 +361,7 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     
 // find linearly indepenpendent submatrix of maximal rank
 
-    vector<key_t>  max_rank_submatrix_lex() const; //returns a vector with entries
+    vector<key_t>  max_rank_submatrix_lex(vector<key_t> perm=vector<key_t>(0)) const; //returns a vector with entries
     //the indices of the first rows in lexicographic order of this forming
     //a submatrix of maximal rank.
     
@@ -430,9 +450,11 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     
 // Sorting of rows
     
+    void order_rows_by_perm(const vector<key_t>& perm);
+    
     Matrix& sort_by_weights(const Matrix<Integer>& Weights, vector<bool> absolute);
     Matrix& sort_lex();
-    void order_rows_by_perm(const vector<key_t>& perm);
+    Matrix<Integer>& sort_by_nr_of_zeroes();
     
 // solve homogeneous congruences
     
@@ -461,11 +483,45 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     Matrix<Integer> LLL_transpose() const;
     
     void GramSchmidt(Matrix<nmz_float>& B, Matrix<nmz_float>& M, int from, int to);
-
+    
+// check maztrix for defining a projection and using it
+    
+    bool check_projection(vector<key_t>& projection_key);
+    Matrix select_coordinates(const vector<key_t>& projection_key) const; // applies the projection
+    Matrix insert_coordinates(const vector<key_t>& projection_key, const size_t nr_cols) const; // defines the "inverse"
 };
 //class end *****************************************************************
 
+
 //---------------------------------------------------------------------------
+//                  Matrices of binary expansions
+//---------------------------------------------------------------------------
+class BinaryMatrix {
+    
+    vector<vector<boost::dynamic_bitset<> > > Layers;
+    size_t nr_rows, nr_columns;
+    bool at_least_one_negative;
+    bool negative_converted_to_layer;
+    mpz_class offset; // to be added to "entries" to get true value
+    
+public:
+    
+    template<typename Integer>
+    void insert(Integer val, key_t i, key_t j);
+    
+    bool test(key_t i, key_t j, key_t k) const;
+    BinaryMatrix();
+    BinaryMatrix(size_t m, size_t n);
+    BinaryMatrix(size_t m,size_t n, size_t height);
+    size_t nr_layers() const;
+    BinaryMatrix reordered(const vector<long>& row_order, const vector<long>& col_order) const;
+    bool equal(const BinaryMatrix& Comp) const;
+    
+    template<typename Integer>
+    void set_offset(Integer M);
+    
+};
+//class end *****************************************************************
 //                  LLL with returned transformation matrices
 //---------------------------------------------------------------------------
 
@@ -596,6 +652,7 @@ void mpz_submatrix(Matrix<mpz_class>& sub, const Matrix<Integer>& mother, const
 template<typename Integer>
 void mpz_submatrix_trans(Matrix<mpz_class>& sub, const Matrix<Integer>& mother, const vector<key_t>& selection);
 
+
 template<typename ToType, typename FromType>
 void convert(Matrix<ToType>& to_mat, const Matrix<FromType>& from_mat){
     size_t nrows = from_mat.nr_of_rows();
